
# q-expansion of newform 3920.2.a.v, downloaded from the LMFDB on 26 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3920
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [1], [-1], [0], [2], [4], [0], [-6], [-3], [-3], [0], [-12], [-7], [9], [0], [-6], [10], [5], [-11], [10], [-8], [-6], [3], [17], [-2], [-17], [15], [1], [-5], [-18], [-8], [0], [-4], [-18], [-17], [-20], [-14], [12], [-3], [-8], [8], [-5], [-14], [-22], [-6], [-12], [14], [4], [4], [-2], [-4], [-20], [18], [4], [24], [-21], [31], [2], [8], [26], [28], [-4], [21], [-10], [-16], [-12], [-4], [10], [25], [17], [18], [24], [-13], [36], [-34], [33], [-14], [-2], [-33], [-13], [0], [-31], [22], [18], [4], [21], [39], [8], [-30], [-29], [33], [-36], [-32], [24], [10], [37], [21], [-18], [-28], [39], [13], [30], [11], [22], [18], [2], [-20], [-10], [12], [2], [7], [8], [-20], [-26], [6], [13], [4], [-45], [0], [46], [-7], [-16], [-36], [-7], [10], [45], [1], [-26], [13], [14], [10], [-9], [-28], [20], [-22], [2], [4], [37], [8], [-11], [-32], [-34], [13], [31], [-14], [-46], [34], [-26], [-56], [43], [-22], [41], [-20], [-37], [-9], [34], [-34], [27], [4], [-38], [13], [-28], [-13], [-40], [18], [-9], [-20], [14], [-35], [54], [24], [7], [36], [18], [56], [-39], [6], [-30], [9], [-1], [17], [36], [-6], [54], [4], [41], [-16], [28], [-41], [-30], [-36], [24], [-2], [13], [28], [18], [-31], [34], [-58], [-8], [9], [-44], [-2], [37], [-36], [-6], [46], [44], [1], [-36], [-2], [41], [15], [39], [42], [18], [29], [-29], [-32], [48], [13], [-46], [33], [24], [-13], [-21], [16], [-18], [-31], [-32], [-36], [18], [-66], [-15], [37], [-5], [-73], [-16], [-22], [66], [60], [-16], [-32], [-15], [-24], [38], [-29], [-24], [16], [24], [16], [7], [-1], [-37], [24], [-54], [-5], [23], [6], [-42], [-45], [-15], [-6], [-56], [72], [-50], [-50], [-50], [-15], [-4], [27], [-80], [46], [-64], [22], [-64], [-24], [-15], [29], [-58], [-23], [32], [-56], [-1], [75], [-12], [-46], [0], [72], [54], [45], [-43], [12], [-50], [14], [3], [14], [-26], [30], [12], [76], [26], [-86], [-79], [0], [76], [4], [-27], [-36], [-56], [15], [-42], [-25], [25], [-63], [-47], [-16], [-2], [14], [-33], [16], [-76], [-19], [-59], [56], [65], [-60], [-73], [88], [10], [-14], [56], [22], [25], [42], [69], [-38], [-62], [21], [-88], [4], [36], [-65], [54], [78], [-54], [65], [49], [-40], [0], [-22], [-8], [13], [7], [25], [-16], [48], [-18], [-28], [49], [-50], [6], [9], [20], [87], [-88], [92], [-56], [-3], [-34], [40], [-21], [-78], [32], [32], [-66], [50], [24], [-42], [-54], [39], [36], [-7], [-64], [68], [59], [-2], [-52], [53], [37], [14], [32], [-12], [44], [58], [26], [70], [25], [-20], [-31], [-21], [-86], [31], [16], [-45], [-70], [4], [-33], [-71], [-28], [-26], [22], [-12], [48], [-26], [-78], [42], [48], [62], [-93], [71], [24], [-13], [-108], [-30], [-62], [76], [63], [-40], [78]]
